# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import AuditEventListResponse
from telnyx._utils import parse_datetime

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAuditEvents:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        audit_event = client.audit_events.list()
        assert_matches_type(AuditEventListResponse, audit_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        audit_event = client.audit_events.list(
            filter={
                "created_after": parse_datetime("2021-01-01T00:00:00Z"),
                "created_before": parse_datetime("2021-01-01T00:00:00Z"),
            },
            page={
                "number": 1,
                "size": 10,
            },
            sort="desc",
        )
        assert_matches_type(AuditEventListResponse, audit_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.audit_events.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        audit_event = response.parse()
        assert_matches_type(AuditEventListResponse, audit_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.audit_events.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            audit_event = response.parse()
            assert_matches_type(AuditEventListResponse, audit_event, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncAuditEvents:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        audit_event = await async_client.audit_events.list()
        assert_matches_type(AuditEventListResponse, audit_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        audit_event = await async_client.audit_events.list(
            filter={
                "created_after": parse_datetime("2021-01-01T00:00:00Z"),
                "created_before": parse_datetime("2021-01-01T00:00:00Z"),
            },
            page={
                "number": 1,
                "size": 10,
            },
            sort="desc",
        )
        assert_matches_type(AuditEventListResponse, audit_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.audit_events.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        audit_event = await response.parse()
        assert_matches_type(AuditEventListResponse, audit_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.audit_events.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            audit_event = await response.parse()
            assert_matches_type(AuditEventListResponse, audit_event, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .calls import (
    CallsResource,
    AsyncCallsResource,
    CallsResourceWithRawResponse,
    AsyncCallsResourceWithRawResponse,
    CallsResourceWithStreamingResponse,
    AsyncCallsResourceWithStreamingResponse,
)
from .texml import (
    TexmlResource,
    AsyncTexmlResource,
    TexmlResourceWithRawResponse,
    AsyncTexmlResourceWithRawResponse,
    TexmlResourceWithStreamingResponse,
    AsyncTexmlResourceWithStreamingResponse,
)
from .accounts import (
    AccountsResource,
    AsyncAccountsResource,
    AccountsResourceWithRawResponse,
    AsyncAccountsResourceWithRawResponse,
    AccountsResourceWithStreamingResponse,
    AsyncAccountsResourceWithStreamingResponse,
)

__all__ = [
    "AccountsResource",
    "AsyncAccountsResource",
    "AccountsResourceWithRawResponse",
    "AsyncAccountsResourceWithRawResponse",
    "AccountsResourceWithStreamingResponse",
    "AsyncAccountsResourceWithStreamingResponse",
    "CallsResource",
    "AsyncCallsResource",
    "CallsResourceWithRawResponse",
    "AsyncCallsResourceWithRawResponse",
    "CallsResourceWithStreamingResponse",
    "AsyncCallsResourceWithStreamingResponse",
    "TexmlResource",
    "AsyncTexmlResource",
    "TexmlResourceWithRawResponse",
    "AsyncTexmlResourceWithRawResponse",
    "TexmlResourceWithStreamingResponse",
    "AsyncTexmlResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from datetime import datetime
from typing_extensions import Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = ["AccessIPAddressListParams", "Filter", "FilterCreatedAt", "FilterCreatedAtDateRangeFilter", "Page"]


class AccessIPAddressListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[ip_source], filter[ip_address], filter[created_at]. Supports
    complex bracket operations for dynamic filtering.
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[number], page[size]
    """


class FilterCreatedAtDateRangeFilter(TypedDict, total=False):
    gt: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter for creation date-time greater than"""

    gte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter for creation date-time greater than or equal to"""

    lt: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter for creation date-time less than"""

    lte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter for creation date-time less than or equal to"""


FilterCreatedAt: TypeAlias = Union[Union[str, datetime], FilterCreatedAtDateRangeFilter]


class FilterTyped(TypedDict, total=False):
    created_at: Annotated[FilterCreatedAt, PropertyInfo(format="iso8601")]
    """Filter by exact creation date-time"""

    ip_address: str
    """Filter by IP address"""

    ip_source: str
    """Filter by IP source"""


Filter: TypeAlias = Union[FilterTyped, Dict[str, object]]


class Page(TypedDict, total=False):
    number: int

    size: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    IPListResponse,
    IPCreateResponse,
    IPDeleteResponse,
    IPUpdateResponse,
    IPRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestIPs:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        ip = client.ips.create(
            ip_address="192.168.0.0",
        )
        assert_matches_type(IPCreateResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        ip = client.ips.create(
            ip_address="192.168.0.0",
            connection_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            port=5060,
        )
        assert_matches_type(IPCreateResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.ips.with_raw_response.create(
            ip_address="192.168.0.0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ip = response.parse()
        assert_matches_type(IPCreateResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.ips.with_streaming_response.create(
            ip_address="192.168.0.0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ip = response.parse()
            assert_matches_type(IPCreateResponse, ip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        ip = client.ips.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(IPRetrieveResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.ips.with_raw_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ip = response.parse()
        assert_matches_type(IPRetrieveResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.ips.with_streaming_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ip = response.parse()
            assert_matches_type(IPRetrieveResponse, ip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.ips.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        ip = client.ips.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            ip_address="192.168.0.0",
        )
        assert_matches_type(IPUpdateResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        ip = client.ips.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            ip_address="192.168.0.0",
            connection_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            port=5060,
        )
        assert_matches_type(IPUpdateResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.ips.with_raw_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            ip_address="192.168.0.0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ip = response.parse()
        assert_matches_type(IPUpdateResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.ips.with_streaming_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            ip_address="192.168.0.0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ip = response.parse()
            assert_matches_type(IPUpdateResponse, ip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.ips.with_raw_response.update(
                id="",
                ip_address="192.168.0.0",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        ip = client.ips.list()
        assert_matches_type(IPListResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        ip = client.ips.list(
            filter={
                "connection_id": "connection_id",
                "ip_address": "192.168.0.0",
                "port": 5060,
            },
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(IPListResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.ips.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ip = response.parse()
        assert_matches_type(IPListResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.ips.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ip = response.parse()
            assert_matches_type(IPListResponse, ip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        ip = client.ips.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(IPDeleteResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.ips.with_raw_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ip = response.parse()
        assert_matches_type(IPDeleteResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.ips.with_streaming_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ip = response.parse()
            assert_matches_type(IPDeleteResponse, ip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.ips.with_raw_response.delete(
                "",
            )


class TestAsyncIPs:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        ip = await async_client.ips.create(
            ip_address="192.168.0.0",
        )
        assert_matches_type(IPCreateResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        ip = await async_client.ips.create(
            ip_address="192.168.0.0",
            connection_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            port=5060,
        )
        assert_matches_type(IPCreateResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ips.with_raw_response.create(
            ip_address="192.168.0.0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ip = await response.parse()
        assert_matches_type(IPCreateResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ips.with_streaming_response.create(
            ip_address="192.168.0.0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ip = await response.parse()
            assert_matches_type(IPCreateResponse, ip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        ip = await async_client.ips.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(IPRetrieveResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ips.with_raw_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ip = await response.parse()
        assert_matches_type(IPRetrieveResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ips.with_streaming_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ip = await response.parse()
            assert_matches_type(IPRetrieveResponse, ip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.ips.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        ip = await async_client.ips.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            ip_address="192.168.0.0",
        )
        assert_matches_type(IPUpdateResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        ip = await async_client.ips.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            ip_address="192.168.0.0",
            connection_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            port=5060,
        )
        assert_matches_type(IPUpdateResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ips.with_raw_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            ip_address="192.168.0.0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ip = await response.parse()
        assert_matches_type(IPUpdateResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ips.with_streaming_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            ip_address="192.168.0.0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ip = await response.parse()
            assert_matches_type(IPUpdateResponse, ip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.ips.with_raw_response.update(
                id="",
                ip_address="192.168.0.0",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        ip = await async_client.ips.list()
        assert_matches_type(IPListResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        ip = await async_client.ips.list(
            filter={
                "connection_id": "connection_id",
                "ip_address": "192.168.0.0",
                "port": 5060,
            },
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(IPListResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ips.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ip = await response.parse()
        assert_matches_type(IPListResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ips.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ip = await response.parse()
            assert_matches_type(IPListResponse, ip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        ip = await async_client.ips.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(IPDeleteResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ips.with_raw_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ip = await response.parse()
        assert_matches_type(IPDeleteResponse, ip, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ips.with_streaming_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ip = await response.parse()
            assert_matches_type(IPDeleteResponse, ip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.ips.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import TypeAlias

from .telnyx_campaign_csp import TelnyxCampaignCsp

__all__ = ["CampaignBuilderCreateResponse"]

CampaignBuilderCreateResponse: TypeAlias = Union[TelnyxCampaignCsp, object]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..types import (
    sub_number_order_list_params,
    sub_number_order_update_params,
    sub_number_order_retrieve_params,
    sub_number_order_update_requirement_group_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.sub_number_order_list_response import SubNumberOrderListResponse
from ..types.sub_number_order_cancel_response import SubNumberOrderCancelResponse
from ..types.sub_number_order_update_response import SubNumberOrderUpdateResponse
from ..types.sub_number_order_retrieve_response import SubNumberOrderRetrieveResponse
from ..types.update_regulatory_requirement_param import UpdateRegulatoryRequirementParam
from ..types.sub_number_order_update_requirement_group_response import SubNumberOrderUpdateRequirementGroupResponse

__all__ = ["SubNumberOrdersResource", "AsyncSubNumberOrdersResource"]


class SubNumberOrdersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SubNumberOrdersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return SubNumberOrdersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SubNumberOrdersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return SubNumberOrdersResourceWithStreamingResponse(self)

    def retrieve(
        self,
        sub_number_order_id: str,
        *,
        filter: sub_number_order_retrieve_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubNumberOrderRetrieveResponse:
        """
        Get an existing sub number order.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[include_phone_numbers]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sub_number_order_id:
            raise ValueError(
                f"Expected a non-empty value for `sub_number_order_id` but received {sub_number_order_id!r}"
            )
        return self._get(
            f"/sub_number_orders/{sub_number_order_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"filter": filter}, sub_number_order_retrieve_params.SubNumberOrderRetrieveParams
                ),
            ),
            cast_to=SubNumberOrderRetrieveResponse,
        )

    def update(
        self,
        sub_number_order_id: str,
        *,
        regulatory_requirements: Iterable[UpdateRegulatoryRequirementParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubNumberOrderUpdateResponse:
        """
        Updates a sub number order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sub_number_order_id:
            raise ValueError(
                f"Expected a non-empty value for `sub_number_order_id` but received {sub_number_order_id!r}"
            )
        return self._patch(
            f"/sub_number_orders/{sub_number_order_id}",
            body=maybe_transform(
                {"regulatory_requirements": regulatory_requirements},
                sub_number_order_update_params.SubNumberOrderUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubNumberOrderUpdateResponse,
        )

    def list(
        self,
        *,
        filter: sub_number_order_list_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubNumberOrderListResponse:
        """
        Get a paginated list of sub number orders.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[status],
              filter[order_request_id], filter[country_code], filter[phone_number_type],
              filter[phone_numbers_count]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/sub_number_orders",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"filter": filter}, sub_number_order_list_params.SubNumberOrderListParams),
            ),
            cast_to=SubNumberOrderListResponse,
        )

    def cancel(
        self,
        sub_number_order_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubNumberOrderCancelResponse:
        """
        Allows you to cancel a sub number order in 'pending' status.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sub_number_order_id:
            raise ValueError(
                f"Expected a non-empty value for `sub_number_order_id` but received {sub_number_order_id!r}"
            )
        return self._patch(
            f"/sub_number_orders/{sub_number_order_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubNumberOrderCancelResponse,
        )

    def update_requirement_group(
        self,
        id: str,
        *,
        requirement_group_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubNumberOrderUpdateRequirementGroupResponse:
        """
        Update requirement group for a sub number order

        Args:
          requirement_group_id: The ID of the requirement group to associate

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/sub_number_orders/{id}/requirement_group",
            body=maybe_transform(
                {"requirement_group_id": requirement_group_id},
                sub_number_order_update_requirement_group_params.SubNumberOrderUpdateRequirementGroupParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubNumberOrderUpdateRequirementGroupResponse,
        )


class AsyncSubNumberOrdersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSubNumberOrdersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSubNumberOrdersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSubNumberOrdersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncSubNumberOrdersResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        sub_number_order_id: str,
        *,
        filter: sub_number_order_retrieve_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubNumberOrderRetrieveResponse:
        """
        Get an existing sub number order.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[include_phone_numbers]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sub_number_order_id:
            raise ValueError(
                f"Expected a non-empty value for `sub_number_order_id` but received {sub_number_order_id!r}"
            )
        return await self._get(
            f"/sub_number_orders/{sub_number_order_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"filter": filter}, sub_number_order_retrieve_params.SubNumberOrderRetrieveParams
                ),
            ),
            cast_to=SubNumberOrderRetrieveResponse,
        )

    async def update(
        self,
        sub_number_order_id: str,
        *,
        regulatory_requirements: Iterable[UpdateRegulatoryRequirementParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubNumberOrderUpdateResponse:
        """
        Updates a sub number order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sub_number_order_id:
            raise ValueError(
                f"Expected a non-empty value for `sub_number_order_id` but received {sub_number_order_id!r}"
            )
        return await self._patch(
            f"/sub_number_orders/{sub_number_order_id}",
            body=await async_maybe_transform(
                {"regulatory_requirements": regulatory_requirements},
                sub_number_order_update_params.SubNumberOrderUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubNumberOrderUpdateResponse,
        )

    async def list(
        self,
        *,
        filter: sub_number_order_list_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubNumberOrderListResponse:
        """
        Get a paginated list of sub number orders.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[status],
              filter[order_request_id], filter[country_code], filter[phone_number_type],
              filter[phone_numbers_count]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/sub_number_orders",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"filter": filter}, sub_number_order_list_params.SubNumberOrderListParams
                ),
            ),
            cast_to=SubNumberOrderListResponse,
        )

    async def cancel(
        self,
        sub_number_order_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubNumberOrderCancelResponse:
        """
        Allows you to cancel a sub number order in 'pending' status.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sub_number_order_id:
            raise ValueError(
                f"Expected a non-empty value for `sub_number_order_id` but received {sub_number_order_id!r}"
            )
        return await self._patch(
            f"/sub_number_orders/{sub_number_order_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubNumberOrderCancelResponse,
        )

    async def update_requirement_group(
        self,
        id: str,
        *,
        requirement_group_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubNumberOrderUpdateRequirementGroupResponse:
        """
        Update requirement group for a sub number order

        Args:
          requirement_group_id: The ID of the requirement group to associate

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/sub_number_orders/{id}/requirement_group",
            body=await async_maybe_transform(
                {"requirement_group_id": requirement_group_id},
                sub_number_order_update_requirement_group_params.SubNumberOrderUpdateRequirementGroupParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubNumberOrderUpdateRequirementGroupResponse,
        )


class SubNumberOrdersResourceWithRawResponse:
    def __init__(self, sub_number_orders: SubNumberOrdersResource) -> None:
        self._sub_number_orders = sub_number_orders

        self.retrieve = to_raw_response_wrapper(
            sub_number_orders.retrieve,
        )
        self.update = to_raw_response_wrapper(
            sub_number_orders.update,
        )
        self.list = to_raw_response_wrapper(
            sub_number_orders.list,
        )
        self.cancel = to_raw_response_wrapper(
            sub_number_orders.cancel,
        )
        self.update_requirement_group = to_raw_response_wrapper(
            sub_number_orders.update_requirement_group,
        )


class AsyncSubNumberOrdersResourceWithRawResponse:
    def __init__(self, sub_number_orders: AsyncSubNumberOrdersResource) -> None:
        self._sub_number_orders = sub_number_orders

        self.retrieve = async_to_raw_response_wrapper(
            sub_number_orders.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            sub_number_orders.update,
        )
        self.list = async_to_raw_response_wrapper(
            sub_number_orders.list,
        )
        self.cancel = async_to_raw_response_wrapper(
            sub_number_orders.cancel,
        )
        self.update_requirement_group = async_to_raw_response_wrapper(
            sub_number_orders.update_requirement_group,
        )


class SubNumberOrdersResourceWithStreamingResponse:
    def __init__(self, sub_number_orders: SubNumberOrdersResource) -> None:
        self._sub_number_orders = sub_number_orders

        self.retrieve = to_streamed_response_wrapper(
            sub_number_orders.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            sub_number_orders.update,
        )
        self.list = to_streamed_response_wrapper(
            sub_number_orders.list,
        )
        self.cancel = to_streamed_response_wrapper(
            sub_number_orders.cancel,
        )
        self.update_requirement_group = to_streamed_response_wrapper(
            sub_number_orders.update_requirement_group,
        )


class AsyncSubNumberOrdersResourceWithStreamingResponse:
    def __init__(self, sub_number_orders: AsyncSubNumberOrdersResource) -> None:
        self._sub_number_orders = sub_number_orders

        self.retrieve = async_to_streamed_response_wrapper(
            sub_number_orders.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            sub_number_orders.update,
        )
        self.list = async_to_streamed_response_wrapper(
            sub_number_orders.list,
        )
        self.cancel = async_to_streamed_response_wrapper(
            sub_number_orders.cancel,
        )
        self.update_requirement_group = async_to_streamed_response_wrapper(
            sub_number_orders.update_requirement_group,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ConferenceListParticipantsParams", "Filter", "Page"]


class ConferenceListParticipantsParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[muted], filter[on_hold], filter[whispering]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[after], page[before], page[limit], page[size], page[number]
    """


class Filter(TypedDict, total=False):
    muted: bool
    """If present, participants will be filtered to those who are/are not muted"""

    on_hold: bool
    """If present, participants will be filtered to those who are/are not put on hold"""

    whispering: bool
    """
    If present, participants will be filtered to those who are whispering or are not
    """


class Page(TypedDict, total=False):
    after: str
    """Opaque identifier of next page"""

    before: str
    """Opaque identifier of previous page"""

    limit: int
    """Limit of records per single page"""

    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .porting_orders_exception_type import PortingOrdersExceptionType

__all__ = ["PortingOrderStatus"]


class PortingOrderStatus(BaseModel):
    details: Optional[List[PortingOrdersExceptionType]] = None
    """A list of 0 or more details about this porting order's status"""

    value: Optional[
        Literal[
            "draft",
            "in-process",
            "submitted",
            "exception",
            "foc-date-confirmed",
            "ported",
            "cancelled",
            "cancel-pending",
        ]
    ] = None
    """The current status of the porting order"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["ExportPortingOrdersCsvReportParam", "Filters"]


class Filters(TypedDict, total=False):
    created_at_gt: Annotated[Union[str, datetime], PropertyInfo(alias="created_at__gt", format="iso8601")]
    """The date and time the porting order was created after."""

    created_at_lt: Annotated[Union[str, datetime], PropertyInfo(alias="created_at__lt", format="iso8601")]
    """The date and time the porting order was created before."""

    customer_reference_in: Annotated[SequenceNotStr[str], PropertyInfo(alias="customer_reference__in")]
    """The customer reference of the porting orders to include in the report."""

    status_in: Annotated[
        List[
            Literal[
                "draft",
                "in-process",
                "submitted",
                "exception",
                "foc-date-confirmed",
                "cancel-pending",
                "ported",
                "cancelled",
            ]
        ],
        PropertyInfo(alias="status__in"),
    ]
    """The status of the porting orders to include in the report."""


class ExportPortingOrdersCsvReportParam(TypedDict, total=False):
    filters: Required[Filters]
    """The filters to apply to the export porting order CSV report."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["EventStatus"]

EventStatus: TypeAlias = Literal["pending", "in_progress", "completed", "failed"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

__all__ = ["RequirementListParams", "Filter", "Page"]


class RequirementListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter for requirements (deepObject style).

    Originally: filter[country_code], filter[phone_number_type], filter[action]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """

    sort: List[
        Literal[
            "created_at",
            "updated_at",
            "country_code",
            "phone_number_type",
            "-created_at",
            "-updated_at",
            "-country_code",
            "-phone_number_type",
        ]
    ]
    """Consolidated sort parameter for requirements (deepObject style).

    Originally: sort[]
    """


class Filter(TypedDict, total=False):
    action: Literal["branded_calling", "ordering", "porting"]
    """Filters requirements to those applying to a specific action."""

    country_code: str
    """
    Filters results to those applying to a 2-character (ISO 3166-1 alpha-2) country
    code
    """

    phone_number_type: Literal["local", "national", "toll_free"]
    """Filters results to those applying to a specific phone_number_type"""


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "RcsSuggestionParam",
    "Action",
    "ActionCreateCalendarEventAction",
    "ActionDialAction",
    "ActionOpenURLAction",
    "ActionViewLocationAction",
    "ActionViewLocationActionLatLong",
    "Reply",
]


class ActionCreateCalendarEventAction(TypedDict, total=False):
    description: str
    """Event description. Maximum 500 characters."""

    end_time: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    start_time: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    title: str
    """Event title. Maximum 100 characters."""


class ActionDialAction(TypedDict, total=False):
    phone_number: Required[str]
    """Phone number in +E.164 format"""


class ActionOpenURLAction(TypedDict, total=False):
    application: Required[Literal["OPEN_URL_APPLICATION_UNSPECIFIED", "BROWSER", "WEBVIEW"]]
    """URL open application, browser or webview."""

    url: Required[str]

    webview_view_mode: Required[Literal["WEBVIEW_VIEW_MODE_UNSPECIFIED", "FULL", "HALF", "TALL"]]

    description: str
    """Accessbility description for webview."""


class ActionViewLocationActionLatLong(TypedDict, total=False):
    latitude: Required[float]
    """The latitude in degrees. It must be in the range [-90.0, +90.0]."""

    longitude: Required[float]
    """The longitude in degrees. It must be in the range [-180.0, +180.0]."""


class ActionViewLocationAction(TypedDict, total=False):
    label: str
    """The label of the pin dropped"""

    lat_long: ActionViewLocationActionLatLong

    query: str
    """query string (Android only)"""


class Action(TypedDict, total=False):
    create_calendar_event_action: ActionCreateCalendarEventAction
    """
    Opens the user's default calendar app and starts the new calendar event flow
    with the agent-specified event data pre-filled.
    """

    dial_action: ActionDialAction
    """
    Opens the user's default dialer app with the agent-specified phone number filled
    in.
    """

    fallback_url: str
    """Fallback URL to use if a client doesn't support a suggested action.

    Fallback URLs open in new browser windows. Maximum 2048 characters.
    """

    open_url_action: ActionOpenURLAction
    """Opens the user's default web browser app to the specified URL."""

    postback_data: str
    """
    Payload (base64 encoded) that will be sent to the agent in the user event that
    results when the user taps the suggested action. Maximum 2048 characters.
    """

    share_location_action: object
    """
    Opens the RCS app's location chooser so the user can pick a location to send
    back to the agent.
    """

    text: str
    """Text that is shown in the suggested action. Maximum 25 characters."""

    view_location_action: ActionViewLocationAction
    """Opens the user's default map app and selects the agent-specified location."""


class Reply(TypedDict, total=False):
    postback_data: str
    """
    Payload (base64 encoded) that will be sent to the agent in the user event that
    results when the user taps the suggested action. Maximum 2048 characters.
    """

    text: str
    """Text that is shown in the suggested reply (maximum 25 characters)"""


class RcsSuggestionParam(TypedDict, total=False):
    action: Action
    """When tapped, initiates the corresponding native action on the device."""

    reply: Reply



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .session_list_0_params import SessionList0Params as SessionList0Params
from .session_list_1_params import SessionList1Params as SessionList1Params
from .session_list_0_response import SessionList0Response as SessionList0Response
from .session_list_1_response import SessionList1Response as SessionList1Response
from .session_retrieve_params import SessionRetrieveParams as SessionRetrieveParams
from .session_retrieve_response import SessionRetrieveResponse as SessionRetrieveResponse
from .action_refresh_client_token_params import ActionRefreshClientTokenParams as ActionRefreshClientTokenParams
from .action_refresh_client_token_response import ActionRefreshClientTokenResponse as ActionRefreshClientTokenResponse
from .session_retrieve_participants_params import SessionRetrieveParticipantsParams as SessionRetrieveParticipantsParams
from .session_retrieve_participants_response import (
    SessionRetrieveParticipantsResponse as SessionRetrieveParticipantsResponse,
)
from .action_generate_join_client_token_params import (
    ActionGenerateJoinClientTokenParams as ActionGenerateJoinClientTokenParams,
)
from .action_generate_join_client_token_response import (
    ActionGenerateJoinClientTokenResponse as ActionGenerateJoinClientTokenResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ElevenLabsVoiceSettingsParam"]


class ElevenLabsVoiceSettingsParam(TypedDict, total=False):
    api_key_ref: str
    """
    The `identifier` for an integration secret
    [/v2/integration_secrets](https://developers.telnyx.com/api/secrets-manager/integration-secrets/create-integration-secret)
    that refers to your ElevenLabs API key. Warning: Free plans are unlikely to work
    with this integration.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .portout_report import PortoutReport
from ..pagination_meta import PaginationMeta

__all__ = ["ReportListResponse"]


class ReportListResponse(BaseModel):
    data: Optional[List[PortoutReport]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import httpx
import pytest
from respx import MockRouter

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    PortingOrderListResponse,
    PortingOrderCreateResponse,
    PortingOrderUpdateResponse,
    PortingOrderRetrieveResponse,
    PortingOrderRetrieveSubRequestResponse,
    PortingOrderRetrieveRequirementsResponse,
    PortingOrderRetrieveExceptionTypesResponse,
    PortingOrderRetrieveAllowedFocWindowsResponse,
)
from telnyx._utils import parse_datetime
from telnyx._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPortingOrders:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        porting_order = client.porting_orders.create(
            phone_numbers=["+13035550000", "+13035550001", "+13035550002"],
        )
        assert_matches_type(PortingOrderCreateResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        porting_order = client.porting_orders.create(
            phone_numbers=["+13035550000", "+13035550001", "+13035550002"],
            customer_group_reference="Group-456",
            customer_reference="Acct 123abc",
        )
        assert_matches_type(PortingOrderCreateResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.porting_orders.with_raw_response.create(
            phone_numbers=["+13035550000", "+13035550001", "+13035550002"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        porting_order = response.parse()
        assert_matches_type(PortingOrderCreateResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.porting_orders.with_streaming_response.create(
            phone_numbers=["+13035550000", "+13035550001", "+13035550002"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            porting_order = response.parse()
            assert_matches_type(PortingOrderCreateResponse, porting_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        porting_order = client.porting_orders.retrieve(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PortingOrderRetrieveResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_with_all_params(self, client: Telnyx) -> None:
        porting_order = client.porting_orders.retrieve(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            include_phone_numbers=True,
        )
        assert_matches_type(PortingOrderRetrieveResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.porting_orders.with_raw_response.retrieve(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        porting_order = response.parse()
        assert_matches_type(PortingOrderRetrieveResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.porting_orders.with_streaming_response.retrieve(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            porting_order = response.parse()
            assert_matches_type(PortingOrderRetrieveResponse, porting_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.porting_orders.with_raw_response.retrieve(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        porting_order = client.porting_orders.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PortingOrderUpdateResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        porting_order = client.porting_orders.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            activation_settings={"foc_datetime_requested": parse_datetime("2021-03-19T10:07:15.527Z")},
            customer_group_reference="customer_group_reference",
            customer_reference="customer_reference",
            documents={
                "invoice": "ce74b771-d23d-4960-81ec-8741b3862146",
                "loa": "64ffb720-04c7-455b-92d6-20fcca92e935",
            },
            end_user={
                "admin": {
                    "account_number": "123abc",
                    "auth_person_name": "Porter McPortersen II",
                    "billing_phone_number": "billing_phone_number",
                    "business_identifier": "abc123",
                    "entity_name": "Porter McPortersen",
                    "pin_passcode": "pin_passcode",
                    "tax_identifier": "1234abcd",
                },
                "location": {
                    "administrative_area": "TX",
                    "country_code": "US",
                    "extended_address": "14th Floor",
                    "locality": "Austin",
                    "postal_code": "78701",
                    "street_address": "600 Congress Avenue",
                },
            },
            messaging={"enable_messaging": True},
            misc={
                "new_billing_phone_number": "new_billing_phone_number",
                "remaining_numbers_action": "disconnect",
                "type": "full",
            },
            phone_number_configuration={
                "billing_group_id": "f1486bae-f067-460c-ad43-73a92848f902",
                "connection_id": "f1486bae-f067-460c-ad43-73a92848f902",
                "emergency_address_id": "f1486bae-f067-460c-ad43-73a92848f902",
                "messaging_profile_id": "f1486bae-f067-460c-ad43-73a92848f901",
                "tags": ["abc", "123"],
            },
            requirement_group_id="DE748D99-06FA-4D90-9F9A-F4B62696BADA",
            requirements=[
                {
                    "field_value": "9787fb5f-cbe5-4de4-b765-3303774ee9fe",
                    "requirement_type_id": "59b0762a-b274-4f76-ac32-4d5cf0272e66",
                }
            ],
            user_feedback={
                "user_comment": "I loved my experience porting numbers with Telnyx",
                "user_rating": 5,
            },
            webhook_url="https://example.com",
        )
        assert_matches_type(PortingOrderUpdateResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.porting_orders.with_raw_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        porting_order = response.parse()
        assert_matches_type(PortingOrderUpdateResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.porting_orders.with_streaming_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            porting_order = response.parse()
            assert_matches_type(PortingOrderUpdateResponse, porting_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.porting_orders.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        porting_order = client.porting_orders.list()
        assert_matches_type(PortingOrderListResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        porting_order = client.porting_orders.list(
            filter={
                "activation_settings": {
                    "fast_port_eligible": True,
                    "foc_datetime_requested": {
                        "gt": "2021-03-25T10:00:00.000Z",
                        "lt": "2021-03-25T10:00:00.000Z",
                    },
                },
                "customer_group_reference": "customer_group_reference",
                "customer_reference": "customer_reference",
                "end_user": {
                    "admin": {
                        "auth_person_name": "auth_person_name",
                        "entity_name": "entity_name",
                    }
                },
                "misc": {"type": "full"},
                "parent_support_key": "parent_support_key",
                "phone_numbers": {
                    "carrier_name": "carrier_name",
                    "country_code": "country_code",
                    "phone_number": {"contains": "contains"},
                },
            },
            include_phone_numbers=True,
            page={
                "number": 1,
                "size": 1,
            },
            sort={"value": "created_at"},
        )
        assert_matches_type(PortingOrderListResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.porting_orders.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        porting_order = response.parse()
        assert_matches_type(PortingOrderListResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.porting_orders.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            porting_order = response.parse()
            assert_matches_type(PortingOrderListResponse, porting_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        porting_order = client.porting_orders.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert porting_order is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.porting_orders.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        porting_order = response.parse()
        assert porting_order is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.porting_orders.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            porting_order = response.parse()
            assert porting_order is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.porting_orders.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_allowed_foc_windows(self, client: Telnyx) -> None:
        porting_order = client.porting_orders.retrieve_allowed_foc_windows(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PortingOrderRetrieveAllowedFocWindowsResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve_allowed_foc_windows(self, client: Telnyx) -> None:
        response = client.porting_orders.with_raw_response.retrieve_allowed_foc_windows(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        porting_order = response.parse()
        assert_matches_type(PortingOrderRetrieveAllowedFocWindowsResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_allowed_foc_windows(self, client: Telnyx) -> None:
        with client.porting_orders.with_streaming_response.retrieve_allowed_foc_windows(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            porting_order = response.parse()
            assert_matches_type(PortingOrderRetrieveAllowedFocWindowsResponse, porting_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve_allowed_foc_windows(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.porting_orders.with_raw_response.retrieve_allowed_foc_windows(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_exception_types(self, client: Telnyx) -> None:
        porting_order = client.porting_orders.retrieve_exception_types()
        assert_matches_type(PortingOrderRetrieveExceptionTypesResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve_exception_types(self, client: Telnyx) -> None:
        response = client.porting_orders.with_raw_response.retrieve_exception_types()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        porting_order = response.parse()
        assert_matches_type(PortingOrderRetrieveExceptionTypesResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_exception_types(self, client: Telnyx) -> None:
        with client.porting_orders.with_streaming_response.retrieve_exception_types() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            porting_order = response.parse()
            assert_matches_type(PortingOrderRetrieveExceptionTypesResponse, porting_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_retrieve_loa_template(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/porting_orders/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/loa_template").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        porting_order = client.porting_orders.retrieve_loa_template(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert porting_order.is_closed
        assert porting_order.json() == {"foo": "bar"}
        assert cast(Any, porting_order.is_closed) is True
        assert isinstance(porting_order, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_retrieve_loa_template_with_all_params(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/porting_orders/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/loa_template").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        porting_order = client.porting_orders.retrieve_loa_template(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            loa_configuration_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert porting_order.is_closed
        assert porting_order.json() == {"foo": "bar"}
        assert cast(Any, porting_order.is_closed) is True
        assert isinstance(porting_order, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_retrieve_loa_template(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/porting_orders/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/loa_template").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        porting_order = client.porting_orders.with_raw_response.retrieve_loa_template(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert porting_order.is_closed is True
        assert porting_order.http_request.headers.get("X-Stainless-Lang") == "python"
        assert porting_order.json() == {"foo": "bar"}
        assert isinstance(porting_order, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_retrieve_loa_template(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/porting_orders/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/loa_template").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        with client.porting_orders.with_streaming_response.retrieve_loa_template(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as porting_order:
            assert not porting_order.is_closed
            assert porting_order.http_request.headers.get("X-Stainless-Lang") == "python"

            assert porting_order.json() == {"foo": "bar"}
            assert cast(Any, porting_order.is_closed) is True
            assert isinstance(porting_order, StreamedBinaryAPIResponse)

        assert cast(Any, porting_order.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_path_params_retrieve_loa_template(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.porting_orders.with_raw_response.retrieve_loa_template(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_requirements(self, client: Telnyx) -> None:
        porting_order = client.porting_orders.retrieve_requirements(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PortingOrderRetrieveRequirementsResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_requirements_with_all_params(self, client: Telnyx) -> None:
        porting_order = client.porting_orders.retrieve_requirements(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(PortingOrderRetrieveRequirementsResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve_requirements(self, client: Telnyx) -> None:
        response = client.porting_orders.with_raw_response.retrieve_requirements(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        porting_order = response.parse()
        assert_matches_type(PortingOrderRetrieveRequirementsResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_requirements(self, client: Telnyx) -> None:
        with client.porting_orders.with_streaming_response.retrieve_requirements(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            porting_order = response.parse()
            assert_matches_type(PortingOrderRetrieveRequirementsResponse, porting_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve_requirements(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.porting_orders.with_raw_response.retrieve_requirements(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_sub_request(self, client: Telnyx) -> None:
        porting_order = client.porting_orders.retrieve_sub_request(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PortingOrderRetrieveSubRequestResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve_sub_request(self, client: Telnyx) -> None:
        response = client.porting_orders.with_raw_response.retrieve_sub_request(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        porting_order = response.parse()
        assert_matches_type(PortingOrderRetrieveSubRequestResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_sub_request(self, client: Telnyx) -> None:
        with client.porting_orders.with_streaming_response.retrieve_sub_request(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            porting_order = response.parse()
            assert_matches_type(PortingOrderRetrieveSubRequestResponse, porting_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve_sub_request(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.porting_orders.with_raw_response.retrieve_sub_request(
                "",
            )


class TestAsyncPortingOrders:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        porting_order = await async_client.porting_orders.create(
            phone_numbers=["+13035550000", "+13035550001", "+13035550002"],
        )
        assert_matches_type(PortingOrderCreateResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        porting_order = await async_client.porting_orders.create(
            phone_numbers=["+13035550000", "+13035550001", "+13035550002"],
            customer_group_reference="Group-456",
            customer_reference="Acct 123abc",
        )
        assert_matches_type(PortingOrderCreateResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.with_raw_response.create(
            phone_numbers=["+13035550000", "+13035550001", "+13035550002"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        porting_order = await response.parse()
        assert_matches_type(PortingOrderCreateResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.with_streaming_response.create(
            phone_numbers=["+13035550000", "+13035550001", "+13035550002"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            porting_order = await response.parse()
            assert_matches_type(PortingOrderCreateResponse, porting_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        porting_order = await async_client.porting_orders.retrieve(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PortingOrderRetrieveResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncTelnyx) -> None:
        porting_order = await async_client.porting_orders.retrieve(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            include_phone_numbers=True,
        )
        assert_matches_type(PortingOrderRetrieveResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.with_raw_response.retrieve(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        porting_order = await response.parse()
        assert_matches_type(PortingOrderRetrieveResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.with_streaming_response.retrieve(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            porting_order = await response.parse()
            assert_matches_type(PortingOrderRetrieveResponse, porting_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.porting_orders.with_raw_response.retrieve(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        porting_order = await async_client.porting_orders.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PortingOrderUpdateResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        porting_order = await async_client.porting_orders.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            activation_settings={"foc_datetime_requested": parse_datetime("2021-03-19T10:07:15.527Z")},
            customer_group_reference="customer_group_reference",
            customer_reference="customer_reference",
            documents={
                "invoice": "ce74b771-d23d-4960-81ec-8741b3862146",
                "loa": "64ffb720-04c7-455b-92d6-20fcca92e935",
            },
            end_user={
                "admin": {
                    "account_number": "123abc",
                    "auth_person_name": "Porter McPortersen II",
                    "billing_phone_number": "billing_phone_number",
                    "business_identifier": "abc123",
                    "entity_name": "Porter McPortersen",
                    "pin_passcode": "pin_passcode",
                    "tax_identifier": "1234abcd",
                },
                "location": {
                    "administrative_area": "TX",
                    "country_code": "US",
                    "extended_address": "14th Floor",
                    "locality": "Austin",
                    "postal_code": "78701",
                    "street_address": "600 Congress Avenue",
                },
            },
            messaging={"enable_messaging": True},
            misc={
                "new_billing_phone_number": "new_billing_phone_number",
                "remaining_numbers_action": "disconnect",
                "type": "full",
            },
            phone_number_configuration={
                "billing_group_id": "f1486bae-f067-460c-ad43-73a92848f902",
                "connection_id": "f1486bae-f067-460c-ad43-73a92848f902",
                "emergency_address_id": "f1486bae-f067-460c-ad43-73a92848f902",
                "messaging_profile_id": "f1486bae-f067-460c-ad43-73a92848f901",
                "tags": ["abc", "123"],
            },
            requirement_group_id="DE748D99-06FA-4D90-9F9A-F4B62696BADA",
            requirements=[
                {
                    "field_value": "9787fb5f-cbe5-4de4-b765-3303774ee9fe",
                    "requirement_type_id": "59b0762a-b274-4f76-ac32-4d5cf0272e66",
                }
            ],
            user_feedback={
                "user_comment": "I loved my experience porting numbers with Telnyx",
                "user_rating": 5,
            },
            webhook_url="https://example.com",
        )
        assert_matches_type(PortingOrderUpdateResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.with_raw_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        porting_order = await response.parse()
        assert_matches_type(PortingOrderUpdateResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.with_streaming_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            porting_order = await response.parse()
            assert_matches_type(PortingOrderUpdateResponse, porting_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.porting_orders.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        porting_order = await async_client.porting_orders.list()
        assert_matches_type(PortingOrderListResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        porting_order = await async_client.porting_orders.list(
            filter={
                "activation_settings": {
                    "fast_port_eligible": True,
                    "foc_datetime_requested": {
                        "gt": "2021-03-25T10:00:00.000Z",
                        "lt": "2021-03-25T10:00:00.000Z",
                    },
                },
                "customer_group_reference": "customer_group_reference",
                "customer_reference": "customer_reference",
                "end_user": {
                    "admin": {
                        "auth_person_name": "auth_person_name",
                        "entity_name": "entity_name",
                    }
                },
                "misc": {"type": "full"},
                "parent_support_key": "parent_support_key",
                "phone_numbers": {
                    "carrier_name": "carrier_name",
                    "country_code": "country_code",
                    "phone_number": {"contains": "contains"},
                },
            },
            include_phone_numbers=True,
            page={
                "number": 1,
                "size": 1,
            },
            sort={"value": "created_at"},
        )
        assert_matches_type(PortingOrderListResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        porting_order = await response.parse()
        assert_matches_type(PortingOrderListResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            porting_order = await response.parse()
            assert_matches_type(PortingOrderListResponse, porting_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        porting_order = await async_client.porting_orders.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert porting_order is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        porting_order = await response.parse()
        assert porting_order is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            porting_order = await response.parse()
            assert porting_order is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.porting_orders.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_allowed_foc_windows(self, async_client: AsyncTelnyx) -> None:
        porting_order = await async_client.porting_orders.retrieve_allowed_foc_windows(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PortingOrderRetrieveAllowedFocWindowsResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_allowed_foc_windows(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.with_raw_response.retrieve_allowed_foc_windows(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        porting_order = await response.parse()
        assert_matches_type(PortingOrderRetrieveAllowedFocWindowsResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_allowed_foc_windows(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.with_streaming_response.retrieve_allowed_foc_windows(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            porting_order = await response.parse()
            assert_matches_type(PortingOrderRetrieveAllowedFocWindowsResponse, porting_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve_allowed_foc_windows(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.porting_orders.with_raw_response.retrieve_allowed_foc_windows(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_exception_types(self, async_client: AsyncTelnyx) -> None:
        porting_order = await async_client.porting_orders.retrieve_exception_types()
        assert_matches_type(PortingOrderRetrieveExceptionTypesResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_exception_types(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.with_raw_response.retrieve_exception_types()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        porting_order = await response.parse()
        assert_matches_type(PortingOrderRetrieveExceptionTypesResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_exception_types(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.with_streaming_response.retrieve_exception_types() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            porting_order = await response.parse()
            assert_matches_type(PortingOrderRetrieveExceptionTypesResponse, porting_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_retrieve_loa_template(self, async_client: AsyncTelnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/porting_orders/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/loa_template").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        porting_order = await async_client.porting_orders.retrieve_loa_template(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert porting_order.is_closed
        assert await porting_order.json() == {"foo": "bar"}
        assert cast(Any, porting_order.is_closed) is True
        assert isinstance(porting_order, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_retrieve_loa_template_with_all_params(
        self, async_client: AsyncTelnyx, respx_mock: MockRouter
    ) -> None:
        respx_mock.get("/porting_orders/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/loa_template").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        porting_order = await async_client.porting_orders.retrieve_loa_template(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            loa_configuration_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert porting_order.is_closed
        assert await porting_order.json() == {"foo": "bar"}
        assert cast(Any, porting_order.is_closed) is True
        assert isinstance(porting_order, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_retrieve_loa_template(self, async_client: AsyncTelnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/porting_orders/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/loa_template").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        porting_order = await async_client.porting_orders.with_raw_response.retrieve_loa_template(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert porting_order.is_closed is True
        assert porting_order.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await porting_order.json() == {"foo": "bar"}
        assert isinstance(porting_order, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_retrieve_loa_template(
        self, async_client: AsyncTelnyx, respx_mock: MockRouter
    ) -> None:
        respx_mock.get("/porting_orders/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/loa_template").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        async with async_client.porting_orders.with_streaming_response.retrieve_loa_template(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as porting_order:
            assert not porting_order.is_closed
            assert porting_order.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await porting_order.json() == {"foo": "bar"}
            assert cast(Any, porting_order.is_closed) is True
            assert isinstance(porting_order, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, porting_order.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_path_params_retrieve_loa_template(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.porting_orders.with_raw_response.retrieve_loa_template(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_requirements(self, async_client: AsyncTelnyx) -> None:
        porting_order = await async_client.porting_orders.retrieve_requirements(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PortingOrderRetrieveRequirementsResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_requirements_with_all_params(self, async_client: AsyncTelnyx) -> None:
        porting_order = await async_client.porting_orders.retrieve_requirements(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(PortingOrderRetrieveRequirementsResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_requirements(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.with_raw_response.retrieve_requirements(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        porting_order = await response.parse()
        assert_matches_type(PortingOrderRetrieveRequirementsResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_requirements(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.with_streaming_response.retrieve_requirements(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            porting_order = await response.parse()
            assert_matches_type(PortingOrderRetrieveRequirementsResponse, porting_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve_requirements(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.porting_orders.with_raw_response.retrieve_requirements(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_sub_request(self, async_client: AsyncTelnyx) -> None:
        porting_order = await async_client.porting_orders.retrieve_sub_request(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PortingOrderRetrieveSubRequestResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_sub_request(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.with_raw_response.retrieve_sub_request(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        porting_order = await response.parse()
        assert_matches_type(PortingOrderRetrieveSubRequestResponse, porting_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_sub_request(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.with_streaming_response.retrieve_sub_request(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            porting_order = await response.parse()
            assert_matches_type(PortingOrderRetrieveSubRequestResponse, porting_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve_sub_request(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.porting_orders.with_raw_response.retrieve_sub_request(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["AutoRespConfig"]


class AutoRespConfig(BaseModel):
    id: str

    country_code: str

    created_at: datetime

    keywords: List[str]

    op: Literal["start", "stop", "info"]

    updated_at: datetime

    resp_text: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.porting_orders import phone_number_configuration_list_params, phone_number_configuration_create_params
from ...types.porting_orders.phone_number_configuration_list_response import PhoneNumberConfigurationListResponse
from ...types.porting_orders.phone_number_configuration_create_response import PhoneNumberConfigurationCreateResponse

__all__ = ["PhoneNumberConfigurationsResource", "AsyncPhoneNumberConfigurationsResource"]


class PhoneNumberConfigurationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PhoneNumberConfigurationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return PhoneNumberConfigurationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PhoneNumberConfigurationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return PhoneNumberConfigurationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        phone_number_configurations: Iterable[phone_number_configuration_create_params.PhoneNumberConfiguration]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberConfigurationCreateResponse:
        """
        Creates a list of phone number configurations.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/porting_orders/phone_number_configurations",
            body=maybe_transform(
                {"phone_number_configurations": phone_number_configurations},
                phone_number_configuration_create_params.PhoneNumberConfigurationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberConfigurationCreateResponse,
        )

    def list(
        self,
        *,
        filter: phone_number_configuration_list_params.Filter | Omit = omit,
        page: phone_number_configuration_list_params.Page | Omit = omit,
        sort: phone_number_configuration_list_params.Sort | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberConfigurationListResponse:
        """
        Returns a list of phone number configurations paginated.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[porting_order.status][in][], filter[porting_phone_number][in][],
              filter[user_bundle_id][in][]

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          sort: Consolidated sort parameter (deepObject style). Originally: sort[value]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/porting_orders/phone_number_configurations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                        "sort": sort,
                    },
                    phone_number_configuration_list_params.PhoneNumberConfigurationListParams,
                ),
            ),
            cast_to=PhoneNumberConfigurationListResponse,
        )


class AsyncPhoneNumberConfigurationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPhoneNumberConfigurationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPhoneNumberConfigurationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPhoneNumberConfigurationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncPhoneNumberConfigurationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        phone_number_configurations: Iterable[phone_number_configuration_create_params.PhoneNumberConfiguration]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberConfigurationCreateResponse:
        """
        Creates a list of phone number configurations.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/porting_orders/phone_number_configurations",
            body=await async_maybe_transform(
                {"phone_number_configurations": phone_number_configurations},
                phone_number_configuration_create_params.PhoneNumberConfigurationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberConfigurationCreateResponse,
        )

    async def list(
        self,
        *,
        filter: phone_number_configuration_list_params.Filter | Omit = omit,
        page: phone_number_configuration_list_params.Page | Omit = omit,
        sort: phone_number_configuration_list_params.Sort | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberConfigurationListResponse:
        """
        Returns a list of phone number configurations paginated.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[porting_order.status][in][], filter[porting_phone_number][in][],
              filter[user_bundle_id][in][]

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          sort: Consolidated sort parameter (deepObject style). Originally: sort[value]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/porting_orders/phone_number_configurations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                        "sort": sort,
                    },
                    phone_number_configuration_list_params.PhoneNumberConfigurationListParams,
                ),
            ),
            cast_to=PhoneNumberConfigurationListResponse,
        )


class PhoneNumberConfigurationsResourceWithRawResponse:
    def __init__(self, phone_number_configurations: PhoneNumberConfigurationsResource) -> None:
        self._phone_number_configurations = phone_number_configurations

        self.create = to_raw_response_wrapper(
            phone_number_configurations.create,
        )
        self.list = to_raw_response_wrapper(
            phone_number_configurations.list,
        )


class AsyncPhoneNumberConfigurationsResourceWithRawResponse:
    def __init__(self, phone_number_configurations: AsyncPhoneNumberConfigurationsResource) -> None:
        self._phone_number_configurations = phone_number_configurations

        self.create = async_to_raw_response_wrapper(
            phone_number_configurations.create,
        )
        self.list = async_to_raw_response_wrapper(
            phone_number_configurations.list,
        )


class PhoneNumberConfigurationsResourceWithStreamingResponse:
    def __init__(self, phone_number_configurations: PhoneNumberConfigurationsResource) -> None:
        self._phone_number_configurations = phone_number_configurations

        self.create = to_streamed_response_wrapper(
            phone_number_configurations.create,
        )
        self.list = to_streamed_response_wrapper(
            phone_number_configurations.list,
        )


class AsyncPhoneNumberConfigurationsResourceWithStreamingResponse:
    def __init__(self, phone_number_configurations: AsyncPhoneNumberConfigurationsResource) -> None:
        self._phone_number_configurations = phone_number_configurations

        self.create = async_to_streamed_response_wrapper(
            phone_number_configurations.create,
        )
        self.list = async_to_streamed_response_wrapper(
            phone_number_configurations.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .url import URL as URL
from .volume import Volume as Volume
from .url_param import URLParam as URLParam
from .tf_phone_number import TfPhoneNumber as TfPhoneNumber
from .request_list_params import RequestListParams as RequestListParams
from .use_case_categories import UseCaseCategories as UseCaseCategories
from .request_create_params import RequestCreateParams as RequestCreateParams
from .request_list_response import RequestListResponse as RequestListResponse
from .request_update_params import RequestUpdateParams as RequestUpdateParams
from .tf_phone_number_param import TfPhoneNumberParam as TfPhoneNumberParam
from .tf_verification_status import TfVerificationStatus as TfVerificationStatus
from .verification_request_egress import VerificationRequestEgress as VerificationRequestEgress
from .verification_request_status import VerificationRequestStatus as VerificationRequestStatus
from .toll_free_verification_entity_type import TollFreeVerificationEntityType as TollFreeVerificationEntityType



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["SimCardRetrieveParams"]


class SimCardRetrieveParams(TypedDict, total=False):
    include_pin_puk_codes: bool
    """When set to true, includes the PIN and PUK codes in the response.

    These codes are used for SIM card security and unlocking purposes. Available for
    both physical SIM cards and eSIMs.
    """

    include_sim_card_group: bool
    """It includes the associated SIM card group object in the response when present."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ...types import verified_number_list_params, verified_number_create_params
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.verified_number_data_wrapper import VerifiedNumberDataWrapper
from ...types.verified_number_list_response import VerifiedNumberListResponse
from ...types.verified_number_create_response import VerifiedNumberCreateResponse

__all__ = ["VerifiedNumbersResource", "AsyncVerifiedNumbersResource"]


class VerifiedNumbersResource(SyncAPIResource):
    @cached_property
    def actions(self) -> ActionsResource:
        return ActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> VerifiedNumbersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return VerifiedNumbersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VerifiedNumbersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return VerifiedNumbersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        phone_number: str,
        verification_method: Literal["sms", "call"],
        extension: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifiedNumberCreateResponse:
        """Initiates phone number verification procedure.

        Supports DTMF extension dialing
        for voice calls to numbers behind IVR systems.

        Args:
          verification_method: Verification method.

          extension: Optional DTMF extension sequence to dial after the call is answered. This
              parameter enables verification of phone numbers behind IVR systems that require
              extension dialing. Valid characters: digits 0-9, letters A-D, symbols \\** and #.
              Pauses: w = 0.5 second pause, W = 1 second pause. Maximum length: 50 characters.
              Only works with 'call' verification method.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/verified_numbers",
            body=maybe_transform(
                {
                    "phone_number": phone_number,
                    "verification_method": verification_method,
                    "extension": extension,
                },
                verified_number_create_params.VerifiedNumberCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerifiedNumberCreateResponse,
        )

    def retrieve(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifiedNumberDataWrapper:
        """
        Retrieve a verified number

        Args:
          phone_number: +E164 formatted phone number.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return self._get(
            f"/verified_numbers/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerifiedNumberDataWrapper,
        )

    def list(
        self,
        *,
        page: verified_number_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifiedNumberListResponse:
        """
        Gets a paginated list of Verified Numbers.

        Args:
          page: Consolidated page parameter (deepObject style). Use page[size] and page[number]
              in the query string. Originally: page[size], page[number]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/verified_numbers",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"page": page}, verified_number_list_params.VerifiedNumberListParams),
            ),
            cast_to=VerifiedNumberListResponse,
        )

    def delete(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifiedNumberDataWrapper:
        """
        Delete a verified number

        Args:
          phone_number: +E164 formatted phone number.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return self._delete(
            f"/verified_numbers/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerifiedNumberDataWrapper,
        )


class AsyncVerifiedNumbersResource(AsyncAPIResource):
    @cached_property
    def actions(self) -> AsyncActionsResource:
        return AsyncActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncVerifiedNumbersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVerifiedNumbersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVerifiedNumbersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncVerifiedNumbersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        phone_number: str,
        verification_method: Literal["sms", "call"],
        extension: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifiedNumberCreateResponse:
        """Initiates phone number verification procedure.

        Supports DTMF extension dialing
        for voice calls to numbers behind IVR systems.

        Args:
          verification_method: Verification method.

          extension: Optional DTMF extension sequence to dial after the call is answered. This
              parameter enables verification of phone numbers behind IVR systems that require
              extension dialing. Valid characters: digits 0-9, letters A-D, symbols \\** and #.
              Pauses: w = 0.5 second pause, W = 1 second pause. Maximum length: 50 characters.
              Only works with 'call' verification method.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/verified_numbers",
            body=await async_maybe_transform(
                {
                    "phone_number": phone_number,
                    "verification_method": verification_method,
                    "extension": extension,
                },
                verified_number_create_params.VerifiedNumberCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerifiedNumberCreateResponse,
        )

    async def retrieve(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifiedNumberDataWrapper:
        """
        Retrieve a verified number

        Args:
          phone_number: +E164 formatted phone number.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return await self._get(
            f"/verified_numbers/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerifiedNumberDataWrapper,
        )

    async def list(
        self,
        *,
        page: verified_number_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifiedNumberListResponse:
        """
        Gets a paginated list of Verified Numbers.

        Args:
          page: Consolidated page parameter (deepObject style). Use page[size] and page[number]
              in the query string. Originally: page[size], page[number]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/verified_numbers",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"page": page}, verified_number_list_params.VerifiedNumberListParams),
            ),
            cast_to=VerifiedNumberListResponse,
        )

    async def delete(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifiedNumberDataWrapper:
        """
        Delete a verified number

        Args:
          phone_number: +E164 formatted phone number.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return await self._delete(
            f"/verified_numbers/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerifiedNumberDataWrapper,
        )


class VerifiedNumbersResourceWithRawResponse:
    def __init__(self, verified_numbers: VerifiedNumbersResource) -> None:
        self._verified_numbers = verified_numbers

        self.create = to_raw_response_wrapper(
            verified_numbers.create,
        )
        self.retrieve = to_raw_response_wrapper(
            verified_numbers.retrieve,
        )
        self.list = to_raw_response_wrapper(
            verified_numbers.list,
        )
        self.delete = to_raw_response_wrapper(
            verified_numbers.delete,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        return ActionsResourceWithRawResponse(self._verified_numbers.actions)


class AsyncVerifiedNumbersResourceWithRawResponse:
    def __init__(self, verified_numbers: AsyncVerifiedNumbersResource) -> None:
        self._verified_numbers = verified_numbers

        self.create = async_to_raw_response_wrapper(
            verified_numbers.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            verified_numbers.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            verified_numbers.list,
        )
        self.delete = async_to_raw_response_wrapper(
            verified_numbers.delete,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        return AsyncActionsResourceWithRawResponse(self._verified_numbers.actions)


class VerifiedNumbersResourceWithStreamingResponse:
    def __init__(self, verified_numbers: VerifiedNumbersResource) -> None:
        self._verified_numbers = verified_numbers

        self.create = to_streamed_response_wrapper(
            verified_numbers.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            verified_numbers.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            verified_numbers.list,
        )
        self.delete = to_streamed_response_wrapper(
            verified_numbers.delete,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        return ActionsResourceWithStreamingResponse(self._verified_numbers.actions)


class AsyncVerifiedNumbersResourceWithStreamingResponse:
    def __init__(self, verified_numbers: AsyncVerifiedNumbersResource) -> None:
        self._verified_numbers = verified_numbers

        self.create = async_to_streamed_response_wrapper(
            verified_numbers.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            verified_numbers.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            verified_numbers.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            verified_numbers.delete,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        return AsyncActionsResourceWithStreamingResponse(self._verified_numbers.actions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["UsageReportListParams", "Page"]


class UsageReportListParams(TypedDict, total=False):
    dimensions: Required[SequenceNotStr[str]]
    """Breakout by specified product dimensions"""

    metrics: Required[SequenceNotStr[str]]
    """Specified product usage values"""

    product: Required[str]
    """Telnyx product"""

    date_range: str
    """A more user-friendly way to specify the timespan you want to filter by.

    More options can be found in the Telnyx API Reference docs.
    """

    end_date: str
    """The end date for the time range you are interested in.

    The maximum time range is 31 days. Format: YYYY-MM-DDTHH:mm:ssZ
    """

    filter: str
    """Filter records on dimensions"""

    format: Literal["csv", "json"]
    """Specify the response format (csv or json).

    JSON is returned by default, even if not specified.
    """

    managed_accounts: bool
    """
    Return the aggregations for all Managed Accounts under the user making the
    request.
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[number], page[size]
    """

    sort: SequenceNotStr[str]
    """Specifies the sort order for results"""

    start_date: str
    """The start date for the time range you are interested in.

    The maximum time range is 31 days. Format: YYYY-MM-DDTHH:mm:ssZ
    """

    authorization_bearer: str
    """Authenticates the request with your Telnyx API V2 KEY"""


class Page(TypedDict, total=False):
    number: int

    size: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["AzureConfigurationData"]


class AzureConfigurationData(BaseModel):
    account_key: Optional[str] = None
    """Azure Blob Storage account key."""

    account_name: Optional[str] = None
    """Azure Blob Storage account name."""

    bucket: Optional[str] = None
    """Name of the bucket to be used to store recording files."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, TypedDict

from .rcs_suggestion_param import RcsSuggestionParam
from .rcs_content_info_param import RcsContentInfoParam

__all__ = ["RcsCardContentParam", "Media"]


class Media(TypedDict, total=False):
    content_info: RcsContentInfoParam

    height: Literal["HEIGHT_UNSPECIFIED", "SHORT", "MEDIUM", "TALL"]
    """The height of the media within a rich card with a vertical layout.

    For a standalone card with horizontal layout, height is not customizable, and
    this field is ignored.
    """


class RcsCardContentParam(TypedDict, total=False):
    description: str
    """Description of the card (at most 2000 characters)"""

    media: Media
    """A media file within a rich card."""

    suggestions: Iterable[RcsSuggestionParam]
    """List of suggestions to include in the card. Maximum 10 suggestions."""

    title: str
    """Title of the card (at most 200 characters)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["CampaignDeactivateResponse"]


class CampaignDeactivateResponse(BaseModel):
    time: float

    message: Optional[str] = None

    record_type: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .shared.metadata import Metadata
from .dynamic_emergency_endpoint import DynamicEmergencyEndpoint

__all__ = ["DynamicEmergencyEndpointListResponse"]


class DynamicEmergencyEndpointListResponse(BaseModel):
    data: Optional[List[DynamicEmergencyEndpoint]] = None

    meta: Optional[Metadata] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ActionValidateParams"]


class ActionValidateParams(TypedDict, total=False):
    country_code: Required[str]
    """The two-character (ISO 3166-1 alpha-2) country code of the address."""

    postal_code: Required[str]
    """The postal code of the address."""

    street_address: Required[str]
    """The primary street address information about the address."""

    administrative_area: str
    """The locality of the address.

    For US addresses, this corresponds to the state of the address.
    """

    extended_address: str
    """
    Additional street address information about the address such as, but not limited
    to, unit number or apartment number.
    """

    locality: str
    """The locality of the address.

    For US addresses, this corresponds to the city of the address.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["InferenceEmbeddingTransferToolParams", "Target", "CustomHeader"]


class Target(BaseModel):
    name: Optional[str] = None
    """The name of the target."""

    to: Optional[str] = None
    """The destination number or SIP URI of the call."""


class CustomHeader(BaseModel):
    name: Optional[str] = None

    value: Optional[str] = None
    """The value of the header.

    Note that we support mustache templating for the value. For example you can use
    `{{#integration_secret}}test-secret{{/integration_secret}}` to pass the value of
    the integration secret.
    """


class InferenceEmbeddingTransferToolParams(BaseModel):
    from_: str = FieldInfo(alias="from")
    """Number or SIP URI placing the call."""

    targets: List[Target]
    """The different possible targets of the transfer.

    The assistant will be able to choose one of the targets to transfer the call to.
    """

    custom_headers: Optional[List[CustomHeader]] = None
    """Custom headers to be added to the SIP INVITE for the transfer command."""

    warm_transfer_instructions: Optional[str] = None
    """
    Natural language instructions for your agent for how to provide context for the
    transfer recipient.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .external_vetting_order_params import ExternalVettingOrderParams as ExternalVettingOrderParams
from .external_vetting_import_params import ExternalVettingImportParams as ExternalVettingImportParams
from .external_vetting_import_response import ExternalVettingImportResponse as ExternalVettingImportResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["PhoneNumberAssignmentByProfileAssignParams"]


class PhoneNumberAssignmentByProfileAssignParams(TypedDict, total=False):
    messaging_profile_id: Required[Annotated[str, PropertyInfo(alias="messagingProfileId")]]
    """
    The ID of the messaging profile that you want to link to the specified campaign.
    """

    campaign_id: Annotated[str, PropertyInfo(alias="campaignId")]
    """The ID of the campaign you want to link to the specified messaging profile.

    If you supply this ID in the request, do not also include a tcrCampaignId.
    """

    tcr_campaign_id: Annotated[str, PropertyInfo(alias="tcrCampaignId")]
    """
    The TCR ID of the shared campaign you want to link to the specified messaging
    profile (for campaigns not created using Telnyx 10DLC services only). If you
    supply this ID in the request, do not also include a campaignId.
    """
